/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "CfdDriver.h"
#include "CfdState.h"
#include "RealCfdDriver.h"
#include "PerfectCfdDriver.h"
#include "CfdDef.h"
#include "drivers/global/DriversCommands.h"

namespace cfd {

const string CfdDriver::TYPE = "cfd";

const std::string CfdDriver::WRITE_PARAMETERS_COMMAND = "writeParam";

/*
 * Constructor
 */
CfdDriver::CfdDriver(const string& aName) {

	driver::Vme::init(aName);

	useScopeStr.init(this, NOSAVE, "use_scope_str");
	useListModeStr.init(this, NOSAVE, "use_listmode_str");
	scopeChannel.init(this, NOSAVE, "scope_channel");
	dppAcqMode.init(this, SAVE, "dpp_acq_mode");
	nbChannels.init(this, SAVE, "nb_channels");
	channels.init(this, SAVE, "channels");

	// CFD parameters
	delay.init(this, SAVE, "delay");
	attenuation.init(this, SAVE, "attenuation");

	// Energy filter parameters
	channelActive.init(this, SAVE, "channel_active");
	channelActiveStr.init(this, NOSAVE, "channel_active_str");
	pulsePolarity.init(this, SAVE, "pulse_polarity");
	pulsePolarityStr.init(this, NOSAVE, "pulse_polarity_str");
	trapezoidRise.init(this, SAVE, "trapezoid_rise");
	trapezoidFlat.init(this, SAVE, "trapezoid_flat");
	decayTime.init(this, SAVE, "decay_time");
	peaking.init(this, SAVE, "peaking");
	bitShift.init(this, SAVE, "bit_shift");

	registerStates(new RealCfdDriver(this), new PerfectCfdDriver(this), new PerfectCfdDriver(this));

	// Init command list
	initCommand(driver::INIT_COMMAND);
	initCommand(driver::READ_INFOS_COMMAND);
	initCommand(WRITE_PARAMETERS_COMMAND);

	// Init functions
	registerFunction(NONE_FUNCTION);

	addressType.update(driver::Vme::A24_ACCESS);
	deviceType = LEAF_DEVICE_TYPE_DEVICE_CONTAINER;

	registerRefresher(channels, &CfdDriver::refreshNbChannelsProperty, this);
	registerRefresher(channelActive, &CfdDriver::refreshChannelActiveProperty, this);
	registerRefresher(pulsePolarity, &CfdDriver::refreshPulsePolarityProperty, this);
	registerRefresher(dppAcqMode, &CfdDriver::refreshDppAcqModeProperty, this);

}

/*
 * Destructor
 */
CfdDriver::~CfdDriver() {
}

/*
 * execute
 */
void CfdDriver::execute(const string& aCommand) {

	commandProgression = PROGRESSION_UNKNOWNSTATE_DEVICE_CONTAINER;
	CfdState* currentState = dynamic_cast<CfdState*>(getCurrentState());

	// Check command
	if (aCommand == driver::INIT_COMMAND) {
		currentState->init();
		commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
	} else if (aCommand == driver::READ_INFOS_COMMAND) {
		// Info command
		currentState->readInfos();
		commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
	} else if (aCommand == WRITE_PARAMETERS_COMMAND) {
		// Write param command
		currentState->writeParam();
		commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
	}
}

/*
 * refreshTotalChannelsProperty
 */
void CfdDriver::refreshNbChannelsProperty(int32 aValue) throw (CannotSetValue) {
	channelActive.resize(aValue);
	channelActiveStr.resize(aValue);
	pulsePolarity.resize(aValue);
	pulsePolarityStr.resize(aValue);
	trapezoidRise.resize(aValue);
	trapezoidFlat.resize(aValue);
	decayTime.resize(aValue);
	peaking.resize(aValue);
	bitShift.resize(aValue);
}

/*
 * refreshDppAcqModeProperty
 */
void CfdDriver::refreshDppAcqModeProperty(int32 value) throw (CannotSetValue) {

	switch (value) {
	case 0:
		useScopeStr.update("Inactive");
		useListModeStr.update("Active");
		break;
	case 1:
		useScopeStr.update("Active");
		useListModeStr.update("Inactive");
		break;
	}
}

/*
 * refreshChannelActiveProperty
 */
void CfdDriver::refreshChannelActiveProperty(int32 index, int32 aValue) throw (CannotSetValue) {

	switch (aValue) {
	case 0:
		channelActiveStr.set(index, "Inactive");
		break;
	case 1:
		channelActiveStr.set(index, "Active");
		break;
	}
}

/*
 * refreshPulsePolarityProperty
 */
void CfdDriver::refreshPulsePolarityProperty(int32 index, int32 aValue) throw (CannotSetValue) {

	switch (aValue) {
	case 0:
		pulsePolarityStr.set(index, "Positive");
		break;
	case 1:
		pulsePolarityStr.set(index, "Negative");
		break;
	}
}

}
